/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	var _ = Nuvola.gettext;
	
	// helper function for bind()
	var toArray = function(obj) {
		return Array.prototype.slice.call(obj);
	}
	
	/**
	 * Binds scope as "this" in the function fn
	 * Better JavaScript engines has this already
	 * implemented as fn.bind(scope).
	 */
	Nuvola.bind = function(scope, fn) {
	    return function () {
	        return fn.apply(scope, toArray(arguments));
	    };
	}
	
	/**
	 * Triggers mouse event on element
	 * 
	 * @param elm Element object
	 * @param name Event name
	 */
	Nuvola.triggerMouseEvent = function(elm, name){
	    var event = document.createEvent('MouseEvents');
	    event.initMouseEvent(name, true, true, document.defaultView, 1, 0, 0, 0, 0, false, false, false, false, 0, elm);
	    elm.dispatchEvent(event);
	}

	/**
	 * Simulates click on element
	 * 
	 * @param elm Element object
	 */
	Nuvola.clickOnElement = function(elm){
	    Nuvola.triggerMouseEvent(elm, 'mouseover');
	    Nuvola.triggerMouseEvent(elm, 'mousedown');
	    Nuvola.triggerMouseEvent(elm, 'mouseup');
	    Nuvola.triggerMouseEvent(elm, 'click');
	}
	
	/**
	 * Creates HTML text node
	 * @param text	text of the node
	 * @return		new text node
	 */
	Nuvola.makeText = function(text){
		return document.createTextNode(text);
	}
	
	/**
	 * Creates HTML element
	 * @param name			element name
	 * @param attributes	element attributes (optional)
	 * @param text			text of the element (optional)
	 * @return				new HTML element
	 */
	Nuvola.makeElement = function(name, attributes, text){
		var elm = document.createElement(name);
		attributes = attributes || {};
		for(var key in attributes){
			elm.setAttribute(key, attributes[key]);
		}
		if(text !== undefined && text !== null){
			elm.appendChild(Nuvola.makeText(text));
		}
		return elm;
	}
	
	/**
	 * Make a JavaScript class.
	 * 
	 * @param parents	parent class or an array of object where the first one is a parent class and others are mixins [can be null]
	 * @param init		a function that initializes an object [can be null]
	 * @return			a constructor of the class
	 */
	Nuvola.makeClass = function(parents, methods){
		var constructor = methods.hasOwnProperty('$init') ? methods.$init : function(){};		
		var parent;
		if(parents){
			if(parents instanceof Array){
				parent = parents.length >= 1 ? parents.shift() : null;
			}
			else{
				parent = parents;
				parents = null;
			}
		}
		else{
			parent = null;
		}
		
		if(parent){
			// Create derived class
			var tmp = function(){};
			tmp.prototype = parent.prototype;
			constructor.prototype = new tmp();
		}
		else{
			parent = Object;
		}
		
		constructor.prototype.$parent = parent.prototype;
		
		if(parents){
			// Add mixins functions to the prototype
			var size = parents.length;
			for(var i = 0; i < size; i++){
				var mixin = parents[i];
				for(var key in mixin){
					var fn = mixin[key];
					if(!fn.name){
						fn.name = key;
					}
					constructor.prototype[key] = fn;
				}
			}
		}
		
		// Add methods
		for(var key in methods){
			var fn = methods[key];
			if(!fn.name){
				fn.name = key;
			}
			constructor.prototype[key] = fn;
		}
		
		return constructor;
	}

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
