/*
	Nuvola Player :: Grooveshark integration
	
	This script provides integration of the GrooveShark streaming service
	on the JavaScript side. It's executed when Grooveshark page is completely
	loaded.

	Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	/**
	 * Creates Google Music integration binded to Nuvola JS API
	 * @param Nuvola Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "GrooveShark";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.timeout = null;
		
		
		/* Periodically try to set the callback */
		this.timeout = setInterval(Nuvola.bind(this, this.setCallback), 100);
	};
	
	/* Set callback function for GrooveShark JS API */
	Integration.prototype.setCallback = function(){
		try{
			window.Grooveshark.setSongStatusCallback(Nuvola.bind(this, this.update));
			clearInterval(this.timeout); // done!
			this.update();
		}
		catch(e){
			// Grooveshark API is probably not finished yet
		}
	}
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(current_song){
		if(!current_song){
			try{
				current_song = window.Grooveshark.getCurrentSongStatus();
			}
			catch(e){
				console.log(e.message);
			}
		}
		if(!current_song) return;
		
		var album_art = null;
		var artist = null;
		var song = null;
		var state = Nuvola.STATE_NONE;
		var album = null;
		var can_prev;
		var can_next;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var can_favorite = false;
		
		try{
			var info = current_song.song;
			if(info){
				song = info.songName;
				album = info.albumName;
				artist = info.artistName;
				album_art = info.artURL;
			}
			
			if(current_song.status == "playing"
			|| current_song.status == "buffering"){
				state = Nuvola.STATE_PLAYING;
			}
			else if(current_song.status == "paused" || info){
				state = Nuvola.STATE_PAUSED;
			}
			else{
				state = Nuvola.STATE_NONE;
			}
		}
		catch(e){
			
		}
		
		// FIXME: check it!
		can_prev = can_next = true;		
		
		this.state = state;
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, can_prev, can_next,
			can_thumbs_up, can_thumbs_down, can_favorite);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			// Grooveshark provides good JavaScript API
			// See http://developers.grooveshark.com/docs/js_api/
			switch(cmd){
			case Nuvola.CMD_TOGGLE:
				if(this.state != Nuvola.STATE_PLAYING){
					window.Grooveshark.play();
				}
				else{
					window.Grooveshark.togglePlayPause();
				}
				break;
			case Nuvola.CMD_PREV_SONG:
				window.Grooveshark.previous();
				break;
			case Nuvola.CMD_PLAY:
				window.Grooveshark.play();
				break;
			case Nuvola.CMD_PAUSE:
				window.Grooveshark.pause();
				break;
			case Nuvola.CMD_NEXT_SONG:
				window.Grooveshark.next();
				break;
			case Nuvola.CMD_THUMBS_UP:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.voteCurrentSong(1);
				break;
			case Nuvola.CMD_THUMBS_DOWN:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.voteCurrentSong(-1);
				break;
			case Nuvola.CMD_FAVORITE:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.favoriteCurrentSong();
				break;
			default:
				// Other actions are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string
			throw (this.name + ": " + e.message);
		}
	}
	
	Nuvola.integration = new Integration(); // Singleton ;-)
})(window._Nuvola);
